>module Useful (
>  hd, tl, member, flatmap, allEqual, (\\), remove, copy, inits, 
>  sortremdups, sortremdupsconcat, remdupsmerge, remdups, labelFront,
>  labelBack, myisAlphanum, spaces, intersection, subset, 
>  multiInstances, listMultiples, diffOneInstance, equalityLists, 
>  replicateConcatInc, replicateListInc)
>where

=========================================================================
Removal of duplicates

>remdups [] = []
>remdups (a:as) = a : remdups (filter (/= a) as)

Sort and remove duplicates

>sortremdups [] = []
>sortremdups [x] = [x]
>sortremdups xs = 
>  remdupsmerge (sortremdups (take n xs)) (sortremdups (drop n xs))
>    where n = div (length xs) 2

Merge two ordered lists without duplicates, removing duplicates

>remdupsmerge :: Ord a => [a] -> [a] -> [a]
>remdupsmerge [] xs = xs
>remdupsmerge xs [] = xs
>remdupsmerge (x:xs) (y:ys)
>  | x < y     = x : remdupsmerge xs (y:ys)
>  | x > y     = y : remdupsmerge (x:xs) ys
>  | x == y    = x : remdupsmerge xs ys

>sortremdupsconcat :: Ord a => [[a]] -> [a]
>sortremdupsconcat = foldr remdupsmerge []

=========================================================================
A few useful functions

>hd = head
>tl = tail

>xs \\ [] = xs
>xs \\ (y:ys) = remove xs y \\ ys

>remove [] y = []
>remove (x:xs) y
>  | x==y        = remove xs y
>  | otherwise   = x:remove xs y

>copy x n = replicate n x

>member a [] = False
>member a (b:bs) = a == b || member a bs

>-- all p = and . map p

>-- any p = or . map p

>flatmap f = concat . map f

>allEqual [] = True
>allEqual (x:xs) = all (== x) xs

Cartesian product

>cp :: [[a]] -> [[a]]
>cp [] = [[]]
>cp (xs:xss) = [y:ys | ys <- cp xss, y <- xs]

All initial segments

>inits [] = [[]]
>inits (x:xs) = [] : map (x:) (inits xs)

Add a common label to the front of each string in a list of strings

>labelFront _ [] = []
>labelFront s (x:xs) = (s++x):labelFront s xs

Add a common label to the end of each string in a list of strings

>labelBack _ [] = []
>labelBack s (x:xs) = (x++s):labelBack s xs

>spaces n = replicate n ' '

Intersection of two lists

>intersection [] l2 = []
>intersection l1 [] = []
>intersection l1 (l:ls) = if member l l1 then l:intersection l1 ls
>			  else intersection l1 ls

Subset definition for lists

>subset l1 l2 = (intersection l1 l2) == l1

MultiInstances l ls returns true if there is more than one instance
of l in ls.

>multiInstances l as = length(as\\[l]) < length as - 1

>listMultiples [] = []
>listMultiples xs = listM xs (remdups xs)
>	where	listM xs [] = []
>		listM xs (y:ys) = if (multiInstances y xs) 
>				  then y:listM xs ys
>				  else listM xs ys

diffOneInstance ls xs removes the elements xs from ls mapping
one-on-one instances.  For example, 

	diffOneInstance [1,1,2,3] [1,2] = [1,3]
whereas
	[1,1,2,3]\\[1,2] = [3]

>diffOneInstance ls [] = ls
>diffOneInstance ls (x:xs) = diffOneInstance (rem x ls) xs
>	where
>		rem x [] = []
>		rem x (l:ls) =	if (x==l) then ls
>				else l:rem x ls


>replicateConcatInc n op str = repInc 1 n op str
>	where
>		repInc count n op str = if (count < n) then 
>					  str++show(count)++op++
>					  repInc (count+1) n op str
>					else str++show(count)

>replicateListInc n str = repInc 1 n str
>	where
>		repInc count n str = if (count <= n) then
>					(str ++ show(count)) :
>					repInc (count+1) n str
>				     else []


Equality of 2 lists in terms of the members, regardless of order and
duplications.  For example, 

	equalityLists [1,2,3] [3,2,1] = True, whereas

	[1,2,3] == [3,2,1] = False.

>equalityLists xs ys = elemMember xs ys && elemMember ys xs

>elemMember xs [] = True
>elemMember xs (y:ys) = member y xs && elemMember xs ys

Note: equalityLists [1,1,1,2,3] [3,2,1] = True, since we do not take
duplicates into account (function mainly used in DataIndep.lhs).


----- For hugs 98 purposes ---------------

>myisAlphanum c = isAlpha c || isDigit c

