package protocols;

import core.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.lang.reflect.Array;
import java.io.ObjectOutputStream;
import java.io.ByteArrayOutputStream;
import java.math.BigInteger;

public class SERVOR extends Protocol {
   
   public SERVOR (ObjectOutputStream oos) {
      super("core.MessageT","core.MessageKeyT","core.NonceT");
      this.oos=oos;
      }
   
   public void run() {
      try {
         IMessageKey kab = null;
         
         println("creating DESkey kab");
         kab = messageKey("DES");
         
         println("waiting for m1b");
         Message m1b = message(get("a",4000));
         checkTag(m1b,"KNNAA)");
         println("Decrypting m1b");
         m1b.decrypt("DES", getSecretKey(nameFromId("a")));
         checkTag(m1b,"NNAA");
         addParts(m1b.getAllFields(),4);
         m0=getPart();
         setValueToNonce(m0.getString(),"na");
         m0=getPart();
         setValueToNonce(m0.getString(),"m");
         m0=getPart();
         setNameToId(m0.getString(),"a");
         m0=getPart();
         setNameToId(m0.getString(),"b");
         
         
         println("waiting for m2");
         Message m2 = message(get("b",4001));
         checkTag(m2,"NAAKNNAA)");
         addParts(m2.getAllFields(),4);
         m0=getPart();
         setValueToNonce(m0.getString(),"m");
         m0=getPart();
         setNameToId(m0.getString(),"a");
         m0=getPart();
         setNameToId(m0.getString(),"b");
         m0=getPart();
         println("Decrypting m0");
         m0.decrypt("DES", getSecretKey(nameFromId("b")));
         checkTag(m0,"NNAA");
         addParts(m0.getAllFields(),4);
         m0=getPart();
         setValueToNonce(m0.getString(),"nb");
         m0=getPart();
         setValueToNonce(m0.getString(),"m");
         m0=getPart();
         setNameToId(m0.getString(),"a");
         m0=getPart();
         setNameToId(m0.getString(),"b");
         
         
         
         println("Building message m3");
         Message m3 =message();
         m3.addMessage(message(nonceFromId("m")));
         m0=message();
         m0.addMessage(message(nonceFromId("nb")));
         m0.addMessage(message(kab.getString()));
         println("Encrypting m0");
         m0.encrypt("DES", getSecretKey(nameFromId("b")));
         m3.addMessage(m0);
         println("Sending message m3");
         put("b", m3.getString());
         
         println("Building message m4");
         Message m4 =message();
         m4.addMessage(message(nonceFromId("m")));
         m0=message();
         m0.addMessage(message(nonceFromId("na")));
         m0.addMessage(message(kab.getString()));
         println("Encrypting m0");
         m0.encrypt("DES", getSecretKey(nameFromId("a")));
         m4.addMessage(m0);
         println("Sending message m4");
         put("a", m4.getString());
         
         
         } catch (Exception e) {e.printStackTrace();System.exit(0);}
      }
   
   public static void subMain(String[] args,ObjectOutputStream oos) {
      SERVOR s;
      while (true) {
         s=new SERVOR(oos);
         args=s.getArgs(args);
         s.newAgent("a",4000);
         s.start();
         }
      }
   
   public String[] getArgs(String[] args) {
      try {
         if (args.length != 2) {
            args = (String[]) Array.newInstance("".getClass(), 2);
            BufferedReader in2 =
            new BufferedReader(new InputStreamReader(System.in));
            println("Type the SecretKeyStore file to use [secretkeystore]");
            args[0]=in2.readLine();
            if (args[0].compareTo("")==0) args[0]="secretkeystore";
            println("Type the name of agent s [s]");
            args[1]=in2.readLine();
            if (args[1].compareTo("")==0) args[1]="s";
            }
         setSKS(args[0]);
         setNameToId(args[1],"s");
         } catch (Exception e) {e.printStackTrace();}
      return args;
      }
   
   public static void main(String[] args) {
      ObjectOutputStream oos=null;
      try {
         ByteArrayOutputStream os = new ByteArrayOutputStream();
         oos=new ObjectOutputStream(os);
         PrintOOS p=new PrintOOS("SERVOR",oos,os);
         p.start();}
      catch (Exception e) {e.printStackTrace();}
      subMain(args,oos);
      }
   }